#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2012 Ethan Zhang<http://github.com/Ethan-Zhang> 
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import time
import os
import fcntl
import struct

from logging.handlers import TimedRotatingFileHandler

class MultiProcessTimedRotatingFileHandler(TimedRotatingFileHandler):

    def doRollover(self):
        """ 
        do a rollover; in this case, a date/time stamp is appended to the filename
        when the rollover happens.  However, you want the file to be named for the
        start of the interval, not the current time.  If there is a backup count,
        then we have to get a list of matching filenames, sort them and remove
        the one with the oldest suffix.
        """
        #if self.stream:
        #    self.stream.close()
        # get the time that this sequence started at and make it a TimeTuple
        t = self.rolloverAt - self.interval
        if self.utc:
            timeTuple = time.gmtime(t)
        else:
            timeTuple = time.localtime(t)
        dfn = self.baseFilename + "." + time.strftime(self.suffix, timeTuple)
        #if os.path.exists(dfn):
        #    os.remove(dfn)
        lockdata = struct.pack('hhllhh', fcntl.F_WRLCK, 0, 0, 0, 0, 0)
        fcntl.fcntl(self.stream, fcntl.F_SETLKW, lockdata)
        if not os.path.exists(dfn) and os.path.exists(self.baseFilename):
            os.rename(self.baseFilename, dfn)
            with open(self.baseFilename, 'a'):
                pass
        if self.backupCount > 0:
            # find the oldest log file and delete it
            #s = glob.glob(self.baseFilename + ".20*")
            #if len(s) > self.backupCount:
            #    s.sort()
            #    os.remove(s[0])
            for s in self.getFilesToDelete():
                os.remove(s)
        #print "%s -> %s" % (self.baseFilename, dfn)
        if self.stream:
            self.stream.close()
        self.mode = 'a'
        self.stream = self._open()
        currentTime = int(time.time())
        newRolloverAt = self.computeRollover(currentTime)
        while newRolloverAt <= currentTime:
            newRolloverAt = newRolloverAt + self.interval
        #If DST changes and midnight or weekly rollover, adjust for this.
        if (self.when == 'MIDNIGHT' or self.when.startswith('W')) and not self.utc:
            dstNow = time.localtime(currentTime)[-1]
            dstAtRollover = time.localtime(newRolloverAt)[-1]
            if dstNow != dstAtRollover:
                if not dstNow:  # DST kicks in before next rollover, so we need to deduct an hour
                    newRolloverAt = newRolloverAt - 3600
                else:           # DST bows out before next rollover, so we need to add an hour
                    newRolloverAt = newRolloverAt + 3600
        self.rolloverAt = newRolloverAt



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2012 Ethan Zhang<http://github.com/Ethan-Zhang>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import os
import time

import tornado.web
from tornado.web import HTTPError
from tornado.httpclient import AsyncHTTPClient

from mownfish.util.log import LOG

from base_handler import BaseHandler 
from mownfish.error import ErrorCode as ECODE
from mownfish.error import ErrorMessage as EMSG
from mownfish.error import BaseError


class StatInfoHandler(BaseHandler):

    def get(self):
        try:
            result = {'code': ECODE.SUCCESS, 'msg': EMSG.SUCCESS}
            stat_info = self.application.stat_info()
            result['stat_info']={}
            result['stat_info']['handlers_n']=stat_info['handlers']
            uptime = stat_info['uptime']
            up_day = uptime // 86400
            up_hour = (uptime - up_day*86400) // 3600
            up_minute = (uptime - up_day*86400 - up_hour*3600) // 60
            up_second = uptime - up_day*86400 - up_hour*3600 - up_minute*60
            result['stat_info']['uptime']="%ddays, %dhours, %dminute, %.3fseconds" %\
                                        (up_day, up_hour, up_minute, up_second)
            self.finish(result)
            self.set_accesslog_item('status', 'SUCCESS')

        except BaseError as e:
            LOG.error(e, exc_info=True)
            self.finish({'code':e.e_code, 'msg': '%s' % e})
        except Exception as e:
            LOG.error(e, exc_info=True)
            self.finish({'code':ECODE.DEFAULT, 'msg':
                'Unknown'})

        finally:
            self.write_accesslog()





#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2012 Ethan Zhang<http://github.com/Ethan-Zhang> 
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import os
import sys
import logging
import logging.handlers

from tornado.options import define, options
from mownfish.util.multiprocesslogging import MultiProcessTimedRotatingFileHandler

LOG = logging.getLogger()

def _setup_logging_from_conf(name):
    if not options.log_path or not name:
        return
    
    if name == 'Main':
        _logger = logging.getLogger()
    else:
        _logger = logging.getLogger(name)

    if options.log_level == 'none':
        return

    if name == 'Access':
        _logger.setLevel(logging.INFO)
    else:
        _logger.setLevel(getattr(logging, options.log_level))
    
    formatter = logging.Formatter('%(asctime)s\t%(levelname)s\t%(process)d\t%(module)s.%(funcName)s:%(lineno)d\t%(message)s', '')
    _log_file = '%s/mownfish.%s.%s.log' % (options.log_path, 
                                            name,
                                            options.port)
    if options.multiports:
        timelog = logging.handlers.TimedRotatingFileHandler(_log_file,
                'midnight', 1, 0)
    else:
        timelog = MultiProcessTimedRotatingFileHandler(_log_file, 'midnight',
                                                    1, 0)
    if name == 'Main':
        timelog.setFormatter(formatter)
    _logger.addHandler(timelog)

def setup(name):
    _setup_logging_from_conf(name)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2012 Ethan Zhang<http://github.com/Ethan-Zhang> 
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import statinfo_handler
import index_handler

ROUTES = { 'mownfish':
                    [(r'/statinfo', statinfo_handler.StatInfoHandler),
                     (r'/', index_handler.IndexHandler),
                     ],

        }



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2012 Ethan Zhang<http://github.com/Ethan-Zhang> 
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from __future__ import print_function
import os
import re
import string
import importlib
import sys
import shutil
import re
import argparse

import mownfish

TEMPLATES_TO_RENDER = (
        ('${project_name}', 'http_server.py.tmpl'),
        ('${project_name}', 'timer_task.py.tmpl'),
        ('${project_name}', 'cmd', 'mownfishd.py.tmpl'),
        ('${project_name}', 'handlers', '__init__.py.tmpl'),
        ('${project_name}', 'handlers', 'base_handler.py.tmpl'),
        ('${project_name}', 'handlers', 'statinfo_handler.py.tmpl'),
        ('${project_name}', 'util', 'config.py.tmpl'),
        ('${project_name}', 'util', 'log.py.tmpl'),
        )
IGNORE = shutil.ignore_patterns('*.pyc', '.svn')

def _is_valid_name(project_name):
    def _module_exists(module_name):
        try:
            importlib.import_module(module_name)
            return True
        except ImportError:
            return False

    if not re.search(r'^[_a-zA-Z]\w*$', project_name):
        print('Error: Project names must begin with a letter and contain'\
                ' only\nletters, numbers and underscores')
    elif os.path.exists(project_name):
        print('Error: Directory %r already exists' % project_name)
    elif _module_exists(project_name):
        print('Error: Module %r already exists' % project_name)
    else:
        return True
    return False

def render_templatefile(path, **kwargs):
    with open(path, 'rb') as fp:
        raw = fp.read().decode('utf-8')

    content = string.Template(raw).substitute(**kwargs)
    render_path = path[:-len('.tmpl')] if path.endswith('.tmpl') else path
    with open(render_path, 'wb') as fp:
        fp.write(content.encode('utf-8'))
    if path.endswith('.tmpl'):
        os.remove(path)

CAMELCASE_INVALID_CHARS = re.compile('[^a-zA-Z\d]')
def string_camelcase(string):
    """ Convert a word  to its CamelCase version and remove invalid chars

    >>> string_camelcase('lost-pound')
    'LostPound'
    
    >>> string_camelcase('missing_images')
    'MissingImages'
    """
    return CAMELCASE_INVALID_CHARS.sub('', string.title())

def main():

    parser = argparse.ArgumentParser(description='create new Tornado project')
    parser.add_argument("project_name", metavar='Name', type=str,
                        help="project name")
    args = parser.parse_args()

    project_name = args.project_name
    if not _is_valid_name(project_name):
        sys.exit(1)

    templates_dir = os.path.join(mownfish.__path__[0], 'templates')
    shutil.copytree(templates_dir, project_name, ignore=IGNORE)
    shutil.move(os.path.join(project_name, 'module'), os.path.join(project_name,
        project_name))
    
    for paths in TEMPLATES_TO_RENDER:
        path = os.path.join(*paths)
        tplfile = os.path.join(project_name,
                string.Template(path).substitute(project_name=project_name))
        render_templatefile(tplfile, project_name=project_name,
                ProjectName=string_camelcase(project_name))
    print("New Scrapy project %r, using template directory %r, created in:" % (project_name, 
                            templates_dir))
    print("    %s" % os.path.abspath(project_name))
    print("You can start your Tornado project with:")
    print("    cd %s" % project_name)
    print("    python %s/cmd/mownfishd.py --port=8086" % project_name)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2012 Ethan Zhang<http://github.com/Ethan-Zhang> 
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

''' tornado web server
'''

import os
import sys
import time
import signal
import socket

import tornado.web
import tornado.httpserver
from tornado.options import define, options

from mownfish.util.log import LOG
import mownfish.handlers

define('multiports', default=True)
define('num_process', default=1)

class TApplication(tornado.web.Application):

    def __init__(self, application_name):
        debug = options.env == "debug"
        app_settings = { 
                'gzip': 'on',
                'static_path': os.path.join(os.path.dirname(__file__),
                            "static"),
                'debug':debug if options.multiports else False,
                }

        self._start_time = time.time()

        tornado.web.Application.__init__(self, 
                                    mownfish.handlers.ROUTES[application_name],
                                    **app_settings)

    def stat_info(self):
        handlers = len(tornado.ioloop.IOLoop.instance()._handlers)
        return {'handlers': handlers, 
                'uptime': (time.time() - self._start_time)}
        

class Server(object):

    def __init__(self, application, prepare, log_list):
        self.application = application
        self.prepare = prepare
        self.log_list = log_list

    def start(self):

        def kill_server(sig, frame):

            LOG.warning( 'Catch SIG: %d' % sig )

            tornado.ioloop.IOLoop.instance().stop()


        # ignore Broken Pipe signal
        signal.signal(signal.SIGPIPE, signal.SIG_IGN);
                            
        # catch kill signal
        signal.signal(signal.SIGINT, kill_server)
        signal.signal(signal.SIGQUIT, kill_server)
        signal.signal(signal.SIGTERM, kill_server)
        signal.signal(signal.SIGHUP, kill_server)

        for log_name in self.log_list:
            mownfish.util.log.setup(log_name)

        LOG.info('START TORNADO WEB SERVER ...')

        version_new = True
        if [int(v_bit) for v_bit in tornado.version.split('.')] <= [2,4,1]:
            version_new = False

        for key, value in sorted(options.items(), key=lambda d:d[0]):
            value = value if version_new else value.value()
            if key not in ('help', 'log_file_prefix', 'log_to_stderr') \
                    and not value:
                sys.stderr.write('must specify %s\n' % key)
                options.print_help()
                sys.exit(0)
            LOG.info('Options: (%s, %s)', key, value)

        try:
            sockets = tornado.netutil.bind_sockets(options.port,
                    address=options.bind_ip,
                    backlog=128)

            if not options.multiports:
                task_id = tornado.process.fork_processes(options.num_process)
            
            http_server =  \
                tornado.httpserver.HTTPServer(xheaders=True,
                                            request_callback=self.application)
            http_server.add_sockets(sockets)

            self.prepare()

            tornado.ioloop.IOLoop.instance().start()

            http_server.stop()
            tornado.ioloop.IOLoop.instance().stop()

            LOG.info('STOP TORNADO WEB SERVER ...')
        except socket.error as e:
            LOG.warning('Socket Error: %s' % str(e))
        except KeyboardInterrupt as e:
            LOG.warning('Gently Quit')
        except Exception as e:
            LOG.error('UnCaught Exception: %s' % e, exc_info=True)



    def hup(self):
        pass




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2012 Ethan Zhang<http://github.com/Ethan-Zhang> 
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import os
import sys

# If ../__init__.py exists, add ../ to Python search path, so that
# it will override what happens to be installed in /usr/(local/)lib/python...
possible_topdir = os.path.normpath(os.path.join(os.path.abspath(sys.argv[0]),
                                   os.pardir,
                                    os.pardir,
                                    os.pardir))
if os.path.exists(os.path.join(possible_topdir, 'mownfish', '__init__.py')):
        sys.path.insert(0, possible_topdir)


import mownfish.util.config
from mownfish.util import log as logging
from mownfish import http_server
import mownfish.timer_task

def prepare():
    task = mownfish.timer_task.TimerTask()
    task.start()

def main():
    mownfish.util.config.init_options()
    log_list = ('Main','Access')

    server = http_server.Server(http_server.TApplication('mownfish'), prepare, log_list)
    server.start()

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
#
# Copyright 2012 Ethan Zhang<http://github.com/Ethan-Zhang> 
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

'''Define Error 
'''

class ErrorCode(object):
    DEFAULT = 0xff
    SUCCESS = 0x00

    PARAMETERS = 100 
    PARAMETERS_EMPTY = 101 
    PARAMETERS_TYPE = 102 
    PARAMETERS_DATA = 103 

    TCP = 500 
    HTTP = 501 

    DB_ERROR = 300 
    DB_EMPTY = 301 

class ErrorMessage(object):
    SUCCESS = 'SUCCESS'
    pass

class BaseError(Exception):
    def __init__(self, msg='Default'):
        self.e_code = ErrorCode.DEFAULT
        self.e_msg = 'Uncaught Exception: %s' % Exception.message

    def __str__(self):
        return self.e_msg 
    pass

class ParameterError(BaseError):
    def __init__(self, msg=''):
        self.e_code = ErrorCode.PARAMETERS
        self.e_msg = 'Invalid Parameter: %s' % msg
        pass

    def __str__(self):
        return self.e_msg 
    pass

class ParameterEmptyError(ParameterError):
    def __init__(self, parameter_name=''):
        self.e_code = ErrorCode.PARAMETERS
        self.e_msg = 'Parameter[%s] Can\'t Be NULL' % parameter_name.upper()
        pass

    def __str__(self):
        return self.e_msg 
    pass

class ParameterTypeError(ParameterError):
    def __init__(self, parameter_name=''):
        self.e_code = ErrorCode.PARAMETERS
        self.e_msg = 'Parameter[%s]\'type Is Invalid' % parameter_name.upper()
        pass

    def __str__(self):
        return self.e_msg 
    pass

class ParameterDataError(ParameterError):
    def __init__(self, msg=''):
        self.e_code = ErrorCode.PARAMETERS_DATA 
        self.e_msg = 'Invalid Date: %s' % msg
        pass

    def __str__(self):
        return self.e_msg 
    pass

class DBError(BaseError):
    def __init__(self, msg=''):
        self.e_code = ErrorCode.DB_ERROR 
        self.e_msg = 'DB Error: %s' % msg
        pass

    def __str__(self):
        return self.e_msg 
    pass

class DBEmpty(DBError):
    def __init__(self, msg=''):
        self.e_code = ErrorCode.DB_EMPTY 
        self.e_msg = 'DB Empty: %s' % msg
        pass

    def __str__(self):
        return self.e_msg 
    pass

class NetworkError(BaseError):
    def __init__(self, msg=''):
        self.e_code = 500
        self.e_msg = 'Network IO Error: %s' % msg

    def __str__(self):
        return self.e_msg

class HttpError(NetworkError):
    def __init__(self, msg=''):
        self.e_code = 502
        self.e_msg = 'Client Http Error: %s' % msg

    def __str__(self):
        return self.e_msg

class ServiceError(NetworkError):
    def __init__(self, msg=''):
        self.e_code = 503
        self.e_msg = 'Client Service Error: %s' % msg

    def __str__(self):
        return self.e_msg



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2012 Ethan Zhang<http://github.com/Ethan-Zhang> 
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

'''
DB code sample

from tornado.options import define, options
import tornadoasyncmemcache
import redis
from sqlalchemy import create_engine
from sqlalchemy import text

define("db_addr_list", type=list, default=['192.168.0.176:19803'])
define("redis_ip", type=str, default='192.168.1.96')
define("redis_port", type=int, default=19821)
define("mysqldb_engine", type=str,
        default='mysql://root:ktep@192.168.0.27:3306/KTEP')

class MemcachedClient(object):
    def __new__(cls):
        if not hasattr(cls, '_instance'):
            cls._instance = \
                tornadoasyncmemcache.ClientPool(options.db_addr_list,
                                                maxclients=100)
        return cls._instance

class RedisClient(object):
    def __new__(cls):
        if not hasattr(cls, '_instance'):
            cls._instance = \
                redis.StrictRedis(host=options.redis_ip,
                                    port=options.redis_port, 
                                    db=0)
        return cls._instance

class MySQLClient(object):
    def __new__(cls):
        if not hasattr(cls, '_instance'):
            cls._instance = \
                    create_engine(options.mysqldb_engine, echo=True)
        return cls._instance

def get_memcached(key, callback):
    MemcachedClient().get(key, callback=callback)

def get_redis(key):
    RedisClient().get(key)

def set_redis(key, value):
    RedisClient().set(key, value)

def get_mysql(param):
    sql="select * from table_name where param=:param";
    result = MySQLClient().execute(text(sql), {'param':param}).fetchall()
    return result
'''



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2012 Ethan Zhang<http://github.com/Ethan-Zhang> 
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import time


def format_timestamp(timestamp):
    time_format = time.strftime('%Y-%m-%d %H:%M:%S',
                                time.localtime(timestamp))
    return time_format

def singleton(cls, *args, **kw):
    instance = {}
    def _singleton():
        if cls not in instance:
            instance[cls] = cls(*args, **kw)
        return instance[cls]
    return _singleton




from setuptools import setup, find_packages
setup(
    name="mownfish",
    description="A Tornado Productional Skeleton",
    author="Ethan-Zhang",
    author_email = 'unpeeled_onion@outlook.com',
    license="http://www.apache.org/licenses/LICENSE-2.0",
    url = 'https://github.com/ethan-zhang/mownfish',
    download_url = 'https://github.com/Ethan-Zhang/mownfish/tree/v3.1',
    keywords = ['tornado'],
    packages=find_packages(),
    include_package_data=True,
    version='3.1.3',
    entry_points = {
        'console_scripts': [
            'fishing = mownfish.cmd.fishing:main',
            ],
        },
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2012 Ethan Zhang<http://github.com/Ethan-Zhang> 
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

'''BaseRequestHandler 
'''

import os
import logging

from tornado.web import RequestHandler
from tornado.web import HTTPError
from tornado.options import define, options

from mownfish.util.log import LOG
from mownfish.error import ParameterEmptyError
from mownfish.error import ParameterTypeError
from mownfish.util import commonutils

class BaseHandler(RequestHandler):

    def __init__(self, *args, **kwargv):
        super(BaseHandler, self).__init__(*args, **kwargv)
        self._logger = logging.getLogger('Access')
        self._log_item = []
        self._default_log_item = {}

    def set_default_headers(self):
        self.set_header('Server', options.server_name)

    def prepare(self):
        self._default_log_item['start_time'] = commonutils.format_timestamp(self.request._start_time)
        self._default_log_item['version'] = 'V1'
        self._default_log_item['ip'] = self.request.remote_ip
        self._default_log_item['url'] = self.request.uri
        self._default_log_item['status'] = 'FAILED'
        self._default_log_item['request_time'] = 0

    def on_connection_close(self):
        LOG.info('connection close.')

    def on_finish(self):
        self.set_accesslog_item('request_time', self.request.request_time())

    def add_accesslog_item(self, item='', style='s'):
        self._log_item.append((item,style))

    def set_accesslog_item(self, item_name, item):
        if item_name in self._default_log_item:
            self._default_log_item[item_name] = item

    def write_accesslog(self):
        default_log = '%s`%s`%s`%s`%s`%.3f' % \
                        (self._default_log_item['start_time'],
                        self._default_log_item['version'],
                        self._default_log_item['ip'],
                        self._default_log_item['url'],
                        self._default_log_item['status'],
                        self._default_log_item['request_time'],)
        access_item_s = '`'.join(['%%%s' % item[1] for item in self._log_item]) % \
                    tuple([item[0] for item in self._log_item])
        self._logger.info('%s`%s' % (default_log,access_item_s))
       
    def _check_argument(self, parameter_name,
            default_value=None, expect_types=()):
        try:
            if not default_value:
                v = self.get_argument(parameter_name)
            else:
                v = self.get_argument(parameter_name, default_value)
        except HTTPError as e:
            raise ParameterEmptyError(parameter_name)

        if expect_types and not isinstance(v, expect_types): 
            raise ParameterTypeError(parameter_name)
        return v



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2012 Ethan Zhang<http://github.com/Ethan-Zhang> 
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

'''
@feature: Config Options
'''

import sys
import os

from tornado.options import define, options, Error

CURRENT_PATH = os.path.dirname(os.path.realpath(__file__))
PROJECT_PATH = os.path.realpath(os.path.join(CURRENT_PATH, '..', '..'))

DEFAULT_OPTIONS_LOG_ROOT_PATH     = \
    os.path.realpath(os.path.join(PROJECT_PATH, 'log'))

define("log_level", default = 'DEBUG', 
        help = "Set Log Level")

define('log_path', default = DEFAULT_OPTIONS_LOG_ROOT_PATH, 
        help = 'Log file stored root path')

define('project_path', default =
        os.path.realpath(os.path.join(os.path.realpath(__file__),'..', '..', '..')),
        help = 'Set the Project Path')

define('cfg_file', default = os.path.join(options.project_path, 'etc',
        'mownfish.conf'),
        help = 'set the config file path')

define("bind_ip", default = '0.0.0.0',
        help = "Run server on a specific IP")

define("port", type = int, 
        help = 'Run server on a specific port')

define("server_name", default = 'MOWNFISH/1.0',
        help = 'set the http server name')

define("env", default="debug", help="service run environment")
    
def _usage():
    print 'Usage: bin/mownfish -log_path=SpecifiedFile -port=SpecifiedPort'
    sys.exit()

def init_options():
    # maybe some options will be use before load config file
    try:
        options.parse_command_line()
    except Error as e:
        print e
        sys.exit()
    options.cfg_file = os.path.abspath(options.cfg_file)
    options.parse_config_file(options.cfg_file)
    if not options.log_path or not options.port:
        options.print_help()
        _usage()
    
    options.log_path = os.path.abspath(options.log_path)

    if not os.path.exists(options.log_path):
        os.makedirs(options.log_path)

